#include "Python.h"
#include "sip.h"
#include "sip/sipAPIdebug_handler.h"
#include "pythoninteropdemo.h"
#include "ui_pythoninteropdemo.h"
#include <QDir>
#include "debug_handler.h"

const char *PythonInteropDemo::CATCH_PYTHON_OUTPUT ="import sys\n"
											"class CatchOutErr():\n"
											"	def __init__(self):\n"
											"		self.value = u''\n"
											"	def write(self, txt):\n"
											"		self.value += unicode(txt)\n"
											"catchOutErr = CatchOutErr()\n"
											"sys.stdout = catchOutErr\n"
											"sys.stderr = catchOutErr\n";
const char *PythonInteropDemo::CATCHER_NAME = "catchOutErr";
const char *PythonInteropDemo::OUTPUT_NAME = "value";
const char *PythonInteropDemo::DEBUG_RUN = "Debug_handler.resume()\n";
const char *PythonInteropDemo::DEBUG_PAUSE = "Debug_handler.pause()\n";
const char *PythonInteropDemo::IMPORT_DEBUG = "from debug_handler import Debug_handler\n";
const char *PythonInteropDemo::APPEND_TO_PATH = "import sys\nsys.path.append(\'";

PythonInteropDemo::PythonInteropDemo(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::PythonInteropDemo)
{
    ui->setupUi(this);
	ui->Editor->setText(tr(APPEND_TO_PATH) + QDir::currentPath() + "\')\n" + IMPORT_DEBUG);
	connect(ui->MenuMainExit, SIGNAL(triggered()), this, SLOT(close()));
	connect(ui->Run, SIGNAL(clicked()), this, SLOT(Run_Clicked()));
	Py_Initialize();
}

PythonInteropDemo::~PythonInteropDemo()
{
	Py_Finalize();
    delete ui;
}

void PythonInteropDemo::Run_Clicked()
{
	ui->Display->clear();
	ui->Run->setEnabled(false);
	QString program = ui->Editor->toPlainText();
	PyObject *name = PyString_FromString("__main__");
	if (name)
	{
		PyObject *module = PyImport_Import(name);
		if (module)
		{
			PyObject *globals = PyModule_GetDict(module);
			if (globals)
			{
				PyRun_SimpleString(CATCH_PYTHON_OUTPUT);
				PyObject *catcher = PyObject_GetAttrString(module, CATCHER_NAME);
				if (catcher)
				{
					PyRun_SimpleString(program.toUtf8());
					PyObject *debug = PyDict_GetItemString(globals, "Debug_handler");
					if (debug)
					{
						sipAPIDef sip_api;
						Debug_handler handler;
						PyObject *sender = sip_api.api_get_pyobject(&handler, sipType_Debug_handler);
						//sipQtAPI qt_api;
						//int success = qt_api.qt_connect(debug_resume_, SIGNAL(triggered()), receiver, "resume", 0);
					}
					PyErr_Print();
					PyObject *output = PyObject_GetAttrString(catcher, OUTPUT_NAME);
					if (output)
					{
						output = PyUnicode_AsUTF8String(output);
						if (output)
						{
							QString output_str = QString::fromUtf8(PyString_AsString(output));
							ui->Display->setText(output_str);
							Py_XDECREF(output);
						}
					}
					else
					{
						ui->Display->setText("Unable to fetch output.");
					}
					Py_XDECREF(catcher);
				}
				else
				{
					ui->Display->setText("Unable to redirect stdout and stderr.");
				}
				Py_XDECREF(globals);
			}
			Py_XDECREF(module);
		}
		Py_XDECREF(name);
	}
	ui->Run->setEnabled(true);
}
